import * as path from 'node:path';
import { homedir } from 'node:os';
import * as readline from 'node:readline/promises';
import { stdin as input, stdout as output } from 'process';
import logger from '../logger.js';
import { file } from '../utils/file.js';
import { read } from '../utils/read.js';
export class ConfigService {
    fileName = '.alia.json';
    defaultConfig = {
        version: 1,
        meta: {
            gist: {
                token: '',
                id: ''
            }
        },
        options: {
            separator: '@',
            shell: false
        },
        alias: {
            test: {
                options: {},
                command: ['echo', 'alia', 'is', 'working!']
            }
        }
    };
    filePath = path.join(homedir(), this.fileName);
    _config;
    get config() {
        if (this._config) {
            return this._config;
        }
        try {
            const config = file.read(this.filePath);
            this._config = JSON.parse(config);
            return this._config;
        }
        catch (e) {
            logger.info('Failed to load config:', this.filePath);
            logger.info('Make sure to run: al --init');
            throw e;
        }
    }
    getAlias(key) {
        return this.config.alias[key];
    }
    setAlias(key, command) {
        this.config.alias[key] = command;
        this.save(this.config);
    }
    removeAlias(key) {
        const { [key]: _, ...rest } = this.config.alias;
        this.config.alias = rest;
        this.save(this.config);
    }
    get separator() {
        return this.config.options.separator;
    }
    set separator(separator) {
        this.config.options.separator = separator;
        this.save(this.config);
    }
    get shell() {
        return this.config.options.shell;
    }
    set shell(value) {
        this.config.options.shell = value;
        this.save(this.config);
    }
    get token() {
        return this.meta.gist.token;
    }
    set token(token) {
        this.meta.gist.token = token;
        this.save(this.config);
    }
    get gistId() {
        return this.meta.gist.id;
    }
    set gistId(id) {
        this.meta.gist.id = id;
        this.save(this.config);
    }
    get meta() {
        return this.config.meta;
    }
    async init() {
        if (file.exists(this.filePath)) {
            const rli = readline.createInterface({ input, output });
            const answer = await read.question(rli, `Config already exists at: ${this.filePath}\nWould you like to overwrite (y/n): `);
            rli.close();
            if (!/y(es)?/.test(answer.toLowerCase())) {
                return;
            }
        }
        this.save(this.defaultConfig);
        logger.info('Created config:', this.filePath);
    }
    save(config) {
        file.write(this.filePath, config);
    }
}
