import { ConfFlag } from '../flags/conf.flag.js';
import { HelpFlag } from '../flags/help.flag.js';
import { InitFlag } from '../flags/init.flag.js';
import { ListFlag } from '../flags/list.flag.js';
import { SetFlag } from '../flags/set.flag.js';
import { SyncFlag } from '../flags/sync.flag.js';
import { RemoveFlag } from '../flags/remove.flag.js';
import { VersionFlag } from '../flags/version.flag.js';
import logger from '../logger.js';
export class FlagService {
    confService;
    gistService;
    flags;
    help;
    constructor(confService, gistService) {
        this.confService = confService;
        this.gistService = gistService;
        this.flags = [ConfFlag, InitFlag, ListFlag, RemoveFlag, SetFlag, SyncFlag, VersionFlag].map((f) => new f(confService, gistService));
        this.help = new HelpFlag(this.confService, this.gistService);
        this.help.run = () => this.showHelp();
        this.flags.push(this.help);
    }
    async run(argv) {
        const [arg, ...args] = argv;
        if (!arg) {
            await this.help.run([], {});
            return true;
        }
        const flag = this.flags.find((f) => this.dashMatch(f.flag, arg));
        if (!flag) {
            return false;
        }
        const dashRegex = /^-{1,2}\w/;
        const cut = flag.flag.key === 'set' ? args.findIndex((a) => a === this.confService.separator) - 1 : undefined;
        let flargs = args.slice(0, cut);
        const mods = flargs.filter((a) => dashRegex.test(a));
        const modData = {};
        for (const modArg of mods) {
            const mod = flag.mods.find((f) => this.dashMatch(f, modArg));
            if (!mod) {
                logger.info(`unknown flag: ${modArg}\navailable options:`);
                this.flagHelp(flag, '');
                return true;
            }
            modData[mod.key] ??= [];
            const start = flargs.findIndex((a) => a === modArg);
            const next = flargs[start + 1];
            const end = mods.includes(next) ? -1 : start + 1;
            const { [start]: key, [end]: value, ...rest } = flargs;
            if (value) {
                modData[mod.key]?.push(value);
            }
            flargs = Object.values(rest);
        }
        return flag.run(args, modData);
    }
    dashMatch(flagLike, value) {
        const dashes = value.match(/^-{1,2}/)?.at(0);
        if (!dashes) {
            return false;
        }
        const v = value.replace(dashes, '');
        const types = {
            '-': 'short',
            '--': 'key'
        };
        const key = types[dashes];
        return flagLike[key] === v;
    }
    showHelp() {
        logger.info(`
    usage:

      $ al [options] [alias] [separator] [command]

    options:
    `);
        this.flags.forEach((flag) => this.flagHelp(flag));
        const separator = this.confService.separator;
        logger.info(`
    examples:

      $ al -s gp ${separator} git push
        > Added: gp ${separator} git push

      $ al gp
        > git push

      $ al -r gp
        > Removed: gp
    `);
        return Promise.resolve(true);
    }
    flagHelp({ flag, mods }, pad = '\t') {
        const desc = (f) => `\t\t${f.desc}`;
        const short = (f) => `, -${f.short}`;
        logger.info(`${pad}--${flag.key}${short(flag)}${desc(flag)}`);
        mods.forEach((m) => {
            logger.info(`${pad}\t--${m.key}${short(m)}${desc(m)}`);
        });
        logger.info();
    }
}
